/*

        󑐋Z@ʃfBXvC̉p

        Copyright 2009-2011, Yoshi Hiroshi.

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "logCommand.h"
#include "simpleUart.h"
#include "switch.h"

unsigned char volatile dummy;
logControl lcb;
unsigned char logBuffer[LOG_DATA_BUFFER_SIZE];

int timebaseTable[] =
{
    150, 200, 500,                          // nS
    1, 2, 5, 10, 20, 50, 100, 200, 500,     // uS
    1, 2, 5, 10, 20, 50, 100, 200, 500,     // mS
    1, 2, 5, 10, 20, 50, 100                // S
};

char timebaseUnitTable[] =
{
    0, 0, 0,
    1, 1, 1, 1, 1, 1, 1, 1, 1,
    2, 2, 2, 2, 2, 2, 2, 2, 2,
    3, 3, 3, 3, 3, 3, 3
};

void initLogCommand()
{
	LOGDATA_PORT_INIT;						// ̓|[g̐ݒ
	LOGSTATUS_LED_DDR_INIT;                 // ԕ\pLED̐ݒ
	BUSY_LED_WAIT_TRIGGER_ON;
    BUSY_LED_CAPTURE_ON;

// init log control block
	lcb.busy = 0;
	lcb.hardwareMask = HARDWARE_BIT_MASK;	// n[hEFA}XN̏i644P,1280xFFj
	lcb.triggerMask = 0x00;					// gK[}XN̏
	lcb.triggerPattern = 0x00;				// gK[p^[̏
	lcb.buff = logBuffer;					// obt@AhX̐ݒ
	lcb.buffSize = LOG_DATA_BUFFER_SIZE;	// obt@TCY̐ݒ
    lcb.buffPtr = lcb.buff + lcb.buffSize;
	lcb.dataReady = 0;

    lcb.sampleSpeedIndex = 1;
    lcb.sampleSpeed = timebaseTable[lcb.sampleSpeedIndex];
    lcb.sampleSpeedUnit = timebaseUnitTable[lcb.sampleSpeedIndex];  // 0:nS, 1:uS, 2:mS
    lcb.timebaseIndex = 3;
    lcb.timebase = timebaseTable[lcb.timebaseIndex];
    lcb.timebaseUnit = timebaseUnitTable[lcb.timebaseIndex];   // 0:nS, 1:uS, 2:mS
    lcb.captureTimer = 0;
    lcb.holdoffTime = 5;
    lcb.captureMode = 0;
    lcb.scrollWidth = 2;    // 0:1div, 1:2div, 2:3div
}

void eraseLogBuffer()
{
    for(unsigned int inx = 0; inx < LOG_DATA_BUFFER_SIZE; inx++) {
        logBuffer[inx] = 0;
    }
}

//**********************************************************
//			Standard Logging Function
//**********************************************************
//	Calling Parameters
//		buffer:
//		interval:
//			 20uS	  20, 1	   50K samples/sec
//			 50uS	  50, 1	   20K samples/sec
//			100uS	 100, 1	   10K samples/sec
//			200uS	 200, 1	    5K samples/sec
//			500uS	 500, 1	    2K samples/sec
//			  1mS	   1, 2     1K samples/sec
//			  2mS	   2, 2    500 samples/sec
//			  5mS	   5, 2    200 samples/sec
//		mask:
//		pattern:
//----------------------------------------------------------

void normalLog()
{
	unsigned char tccr1bTemp;
	unsigned char temp;
	lcb.dataReady = 0;
	lcb.logCounter = 1;
	lcb.busyCounter = 0;
    if(lcb.sampleSpeedUnit < 2) {
	    lcb.busyCounterMax = 40000L / (lcb.sampleSpeed);
    	NORMAL_LOG_OCRA_SET(((lcb.sampleSpeed) * 20) - 1);
        tccr1bTemp = _BV(WGM12) | _BV(CS10);
    } else {
        if(lcb.sampleSpeed < 10) {
    	    lcb.busyCounterMax = (40000L/8) / (lcb.sampleSpeed * 1000L);
        	NORMAL_LOG_OCRA_SET(((lcb.sampleSpeed * (1000L/8)) * 20) - 1);
            tccr1bTemp = _BV(WGM12) | _BV(CS11);
        } else {
    	    lcb.busyCounterMax = (40000L/64) / (lcb.sampleSpeed * 1000L);
        	NORMAL_LOG_OCRA_SET((((lcb.sampleSpeed / 2) * (1000L/8)) * (20/4)) - 1);
            tccr1bTemp = _BV(WGM12) | _BV(CS11) | _BV(CS10);
        }
    }
	NORMAL_LOG_TCNT_SET(69);
	NORMAL_LOG_ENABLE_INTERRUPT;
	if(lcb.triggerMask) BUSY_LED_WAIT_TRIGGER_ON;
	lcb.buffPtr = lcb.buff;
	lcb.logCounter = lcb.buffSize - 1;
	lcb.busy = 1;
    lcb.breakMask = 0xFF;
	LOG_STOP_INT_ENABLE;
	while((((temp = LOGDATA_PIN) ^ lcb.triggerPattern) & lcb.triggerMask & lcb.breakMask) != 0);	// wait for start
	NORMAL_LOG_TIMER_START(tccr1bTemp);
	BUSY_LED_WAIT_TRIGGER_OFF;

	*lcb.buffPtr++ = temp;
	set_sleep_mode(SLEEP_MODE_IDLE);
	if(lcb.logCounter < 10) lcb.dataReady = 1;
	while(!lcb.dataReady)
	{
		sleep_enable();
		sleep_cpu();
		sleep_disable();
	};
	lcb.busy = 0;
	NORMAL_LOG_TCCRB = 0;
	NORMAL_LOG_TIMSK = 0;
	//BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//---------------------------------------------
//		^C}[rvݏ
//---------------------------------------------
ISR(TIMER1_COMPA_vect)
{
	*lcb.buffPtr++ = LOGDATA_PIN;
	--lcb.logCounter;
	if(lcb.logCounter <= 0)
	{
		lcb.dataReady = 1;
	}
	else
	{
		if(lcb.busyCounter++ >= lcb.busyCounterMax)
		{
			lcb.busyCounter = 0;
			BUSY_LED_CAPTURE_INV;
		}
	}
}

////////////////////////////////////////////////////////////

//**********************************************************
//			Middle Speed Logging Function ( 10uS sampling )
//				200 Clock Cycle / 20MHz
//**********************************************************
void Log_200_Cycle()
{
	if(lcb.triggerMask) BUSY_LED_WAIT_TRIGGER_ON;
	unsigned char temp;
	lcb.logCounter = lcb.buffSize - 2;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	lcb.busy = 1;
    lcb.breakMask = 0xFF;
	LOG_STOP_INT_ENABLE;
	while((((temp = LOGDATA_PIN) ^ lcb.triggerPattern) & lcb.triggerMask & lcb.breakMask) != 0);	// wait for start

	LOG_WAIT_186_CYCLE;
	*buff++ = temp;
	do {
		LOG_DATA_1_BYTE_3_CYCLE(buff);
		LOG_WAIT_174_CYCLE;
	} while(--lcb.logCounter >= 0);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Middle Speed Logging Function ( 5uS sampling )
//				100 Clock Cycle / 20MHz
//**********************************************************
void Log_100_Cycle()
{
	if(lcb.triggerMask) BUSY_LED_WAIT_TRIGGER_ON;
	unsigned char temp;
	lcb.logCounter = lcb.buffSize - 2;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	lcb.busy = 1;
    lcb.breakMask = 0xFF;
	LOG_STOP_INT_ENABLE;
	while((((temp = LOGDATA_PIN) ^ lcb.triggerPattern) & lcb.triggerMask & lcb.breakMask) != 0);	// wait for start

	LOG_WAIT_86_CYCLE;
	*buff++ = temp;
	do {
		LOG_DATA_1_BYTE_3_CYCLE(buff);
		LOG_WAIT_74_CYCLE;
	} while(--lcb.logCounter >= 0);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Middle Speed Logging Function ( 2uS sampling )
//				40 Clock Cycle / 20MHz
//**********************************************************
void Log_40_Cycle()
{
	if(lcb.triggerMask) BUSY_LED_WAIT_TRIGGER_ON;
	unsigned char temp;
	lcb.logCounter = lcb.buffSize - 2;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	lcb.busy = 1;
    lcb.breakMask = 0xFF;
	LOG_STOP_INT_ENABLE;
	while((((temp = LOGDATA_PIN) ^ lcb.triggerPattern) & lcb.triggerMask & lcb.breakMask) != 0);	// wait for start

	LOG_WAIT_26_CYCLE;

	*buff++ = temp;
	do {
		LOG_DATA_1_BYTE_3_CYCLE(buff);
		LOG_WAIT_14_CYCLE;
	} while(--lcb.logCounter >= 0);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Middle Speed Logging Function ( 1uS sampling )
//				20 Clock Cycle / 20MHz
//**********************************************************
void Log_20_Cycle()
{
	if(lcb.triggerMask) BUSY_LED_WAIT_TRIGGER_ON;
	lcb.busy = 1;
    lcb.breakMask = 0xFF;
	LOG_STOP_INT_ENABLE;
	unsigned char temp;
	register volatile int logCounter asm("r26") = lcb.buffSize - 2;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	while((((temp = LOGDATA_PIN) ^ lcb.triggerPattern) & lcb.triggerMask & lcb.breakMask) != 0);	// wait for start
	LOG_WAIT_7_CYCLE;
	*buff++ = temp;
	do {
		LOG_DATA_1_BYTE_3_CYCLE(buff);
		LOG_WAIT_13_CYCLE;
	} while(--logCounter >= 0);
	lcb.busy = 0;
    lcb.logCounter = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Middle Speed Logging Function ( 500nS sampling )
//				10 Clock Cycle / 20MHz
//**********************************************************
void Log_10_Cycle()
{
	if(lcb.triggerMask) BUSY_LED_WAIT_TRIGGER_ON;
	lcb.busy = 1;
    lcb.breakMask = 0xFF;
	LOG_STOP_INT_ENABLE;
	register volatile int logCounter asm("r26") = lcb.buffSize - 1;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	while((LOGDATA_PIN ^ lcb.triggerPattern) & lcb.triggerMask & lcb.breakMask);	// wait for start
	do {
		LOG_DATA_1_BYTE_3_CYCLE(buff);
		LOG_WAIT_3_CYCLE;
	} while(--logCounter >= 0);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Faster Logging Function
//				4 Clock Cycle
//**********************************************************
//			20MHz	0.2uS			5M samples/sec


void Log_4_Cycle()
{
	if(lcb.triggerMask) BUSY_LED_WAIT_TRIGGER_ON;
	lcb.busy = 1;
    lcb.breakMask = 0xFF;
	LOG_STOP_INT_ENABLE;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	while(((LOGDATA_PIN ^ lcb.triggerPattern) & lcb.triggerMask & lcb.breakMask) != 0);	// wait for start
	LOG_DATA_1_5K_BYTE_4(buff);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Fastest Logging Function
//				3 Clock Cycle
//**********************************************************
//			20MHz	0.15uS			6.6M samples/sec
/*
void Log_3_Cycle()
{
	if(lcb.triggerMask) BUSY_LED_WAIT_TRIGGER_ON;
	lcb.busy = 1;
    lcb.breakMask = 0xFF;
	LOG_STOP_INT_ENABLE;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	while(((LOGDATA_PIN ^ lcb.triggerPattern) & lcb.triggerMask) != 0);	// wait for start
	LOG_DATA_1_5K_BYTE_3(buff);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}
*/
