/*

        󑐋Z@ʃfBXvC̉p

        Copyright 2009-2011, Yoshi Hiroshi.

*/

#define LOG_DATA_BUFFER_SIZE		(256 * 6)
#define HARDWARE_BIT_MASK			(0x3F)
#define COMMAND_BUFFER_SIZE			20

// PIN ASSIGNMENT
#define LOGDATA_PIN					PINC
#define LOGDATA_DDR					DDRC
#define LOGDATA_PORT				PORTC
#define LOGDATA_PORT_INIT			{LOGDATA_DDR &= ~HARDWARE_BIT_MASK;LOGDATA_PORT = HARDWARE_BIT_MASK;}

// LED PIN ASSIGNMENT
#define LOGSTATUS_LED_PORT			PORTD
#define LOGSTATUS_LED_DDR			DDRD
#define LOGSTATUS_LED_WAIT			2
#define LOGSTATUS_LED_CAPTURE       3
#define LOGSTATUS_LED_DDR_INIT		LOGSTATUS_LED_DDR |= (_BV(LOGSTATUS_LED_WAIT) | _BV(LOGSTATUS_LED_CAPTURE))
#define BUSY_LED_WAIT_TRIGGER_OFF   LOGSTATUS_LED_PORT &= ~_BV(LOGSTATUS_LED_WAIT)
#define BUSY_LED_WAIT_TRIGGER_ON	LOGSTATUS_LED_PORT |= _BV(LOGSTATUS_LED_WAIT)
#define BUSY_LED_WAIT_TRIGGER_INV	LOGSTATUS_LED_PORT ^= _BV(LOGSTATUS_LED_WAIT)
#define BUSY_LED_CAPTURE_OFF        LOGSTATUS_LED_PORT &= ~_BV(LOGSTATUS_LED_CAPTURE)
#define BUSY_LED_CAPTURE_ON	        LOGSTATUS_LED_PORT |= _BV(LOGSTATUS_LED_CAPTURE)
#define BUSY_LED_CAPTURE_INV	    LOGSTATUS_LED_PORT ^= _BV(LOGSTATUS_LED_CAPTURE)

#define LOG_STOP_INT_ENABLE			enableKeyBreak(2);
#define LOG_STOP_INT_DISABLE		enableKeyBreak(0);

#define NORMAL_LOG_OCIEA				OCIE1A
#define NORMAL_LOG_TCNT_SET(val)	NORMAL_LOG_TCNT = (val)
#define NORMAL_LOG_OCRA_SET(val)	NORMAL_LOG_OCRA  = (val)
#define NORMAL_LOG_TIMER_START(crb)		NORMAL_LOG_TCCRA = 0;NORMAL_LOG_TCCRB = (crb)
#define NORMAL_LOG_ENABLE_INTERRUPT	NORMAL_LOG_TIMSK |= _BV(NORMAL_LOG_OCIEA)	
#define IS_LOG_BUSY					(lcb.busy != 0) 


#define NORMAL_LOG_TCNT				TCNT1
#define NORMAL_LOG_TCCRA			TCCR1A
#define NORMAL_LOG_TCCRB			TCCR1B
#define NORMAL_LOG_OCRA				OCR1A

#define NORMAL_LOG_TIMSK			TIMSK1


#define SAMPLE_SPEED_INDEX_LOW          1
#define SAMPLE_SPEED_INDEX_MAX          20
#define TIMEBASE_INDEX_LOW              2
#define TIMEBASE_INDEX_MAX              27

/*
--------------------------------------------------
*/
typedef struct _logControl
{
	volatile unsigned char busy;
	unsigned char hardwareMask;
	volatile unsigned char triggerMask;
	volatile unsigned char triggerPattern;
    volatile unsigned char breakMask;

	unsigned char* buff;
	volatile unsigned char* buffPtr;
	unsigned int  buffSize;
	volatile int  logCounter;
	volatile unsigned char dataReady;
	volatile long busyCounter;
	volatile long busyCounterMax;

    volatile unsigned char sampleSpeedIndex;
    volatile int sampleSpeed;
    volatile char sampleSpeedUnit;  // 0:nS, 1:uS, 2:mS
    volatile unsigned char timebaseIndex;
    volatile int timebase;
    volatile char timebaseUnit;  // 0:nS, 1:uS, 2:mS

    volatile int captureTimer;
    volatile unsigned char holdoffTime;
    volatile char captureMode;
    volatile unsigned char scrollWidth; // 0:1div, 1:2div, 3:2div
} logControl;

extern logControl lcb;
extern unsigned char logBuffer[LOG_DATA_BUFFER_SIZE];
extern int timebaseTable[];
extern char timebaseUnitTable[];

/*
--------------------------------------------------
*/
void initLogCommand();
void eraseLogBuffer();

void normalLog();

// 200 Clock Cycle		10uS
void Log_200_Cycle();

// 100 Clock Cycle		5uS
void Log_100_Cycle();

// 40 Clock Cycle		2uS
void Log_40_Cycle();

// 20 Clock Cycle		1uS
void Log_20_Cycle();

// 10 Clock Cycle		0.5uS
void Log_10_Cycle();

// 4 Clock Cycle		200nS
void Log_4_Cycle();

/*
// 3 Clock Cycle		150nS
void Log_3_Cycle();
*/

/*
--------------------------------------------------
*/
#define LOG_DATA_1_BYTE_3_CYCLE(ptr)	asm volatile("in __tmp_reg__,%2\n\tST Z+,__tmp_reg__":"=z"(ptr):"0"(ptr),"I"(_SFR_IO_ADDR(LOGDATA_PIN)))
#define LOG_DATA_1_BYTE_4_CYCLE(ptr)	asm volatile("in __tmp_reg__,%2\n\tST Z+,__tmp_reg__\n\tnop":"=z"(ptr):"0"(ptr),"I"(_SFR_IO_ADDR(LOGDATA_PIN)))

#define LOG_WAIT_3n_CYCLE(n)			asm volatile("ldi r24, %0\n\t" "1:dec r24\n\t" "brne 1b"::"M"(n));
#define LOG_WAIT_1_CYCLE				asm volatile("nop")
#define LOG_WAIT_2_CYCLE				LOG_WAIT_1_CYCLE;LOG_WAIT_1_CYCLE
#define LOG_WAIT_3_CYCLE				LOG_WAIT_3n_CYCLE(1)
#define LOG_WAIT_4_CYCLE				LOG_WAIT_3n_CYCLE(1); LOG_WAIT_1_CYCLE
#define LOG_WAIT_5_CYCLE				LOG_WAIT_3n_CYCLE(1); LOG_WAIT_2_CYCLE
#define LOG_WAIT_6_CYCLE				LOG_WAIT_3n_CYCLE(2)
#define LOG_WAIT_7_CYCLE				LOG_WAIT_3n_CYCLE(2); LOG_WAIT_1_CYCLE
#define LOG_WAIT_8_CYCLE				LOG_WAIT_3n_CYCLE(2); LOG_WAIT_2_CYCLE
#define LOG_WAIT_13_CYCLE				LOG_WAIT_3n_CYCLE(4); LOG_WAIT_1_CYCLE
#define LOG_WAIT_14_CYCLE				LOG_WAIT_3n_CYCLE(4); LOG_WAIT_2_CYCLE
#define LOG_WAIT_18_CYCLE				LOG_WAIT_3n_CYCLE(6)
#define LOG_WAIT_26_CYCLE				LOG_WAIT_3n_CYCLE(8); LOG_WAIT_2_CYCLE
#define LOG_WAIT_40_CYCLE				LOG_WAIT_3n_CYCLE(13);LOG_WAIT_1_CYCLE
#define LOG_WAIT_53_CYCLE				LOG_WAIT_3n_CYCLE(17);LOG_WAIT_2_CYCLE
#define LOG_WAIT_65_CYCLE				LOG_WAIT_3n_CYCLE(21);LOG_WAIT_2_CYCLE
#define LOG_WAIT_74_CYCLE				LOG_WAIT_3n_CYCLE(24);LOG_WAIT_2_CYCLE
#define LOG_WAIT_86_CYCLE				LOG_WAIT_3n_CYCLE(28);LOG_WAIT_2_CYCLE
#define LOG_WAIT_133_CYCLE				LOG_WAIT_3n_CYCLE(44);LOG_WAIT_1_CYCLE
#define LOG_WAIT_145_CYCLE				LOG_WAIT_3n_CYCLE(48);LOG_WAIT_1_CYCLE
#define LOG_WAIT_174_CYCLE				LOG_WAIT_3n_CYCLE(58)
#define LOG_WAIT_186_CYCLE				LOG_WAIT_3n_CYCLE(62)

////////////////////////////////////////////////////////////

//**********************************************************
//			Fastest Logging Function (3 clock cycle)
//**********************************************************

#define LOG_DATA_2_BYTE_3(ptr)			LOG_DATA_1_BYTE_3_CYCLE(ptr);  LOG_DATA_1_BYTE_3_CYCLE(ptr)
#define LOG_DATA_4_BYTE_3(ptr)			LOG_DATA_2_BYTE_3(ptr);  LOG_DATA_2_BYTE_3(ptr)
#define LOG_DATA_8_BYTE_3(ptr)			LOG_DATA_4_BYTE_3(ptr);  LOG_DATA_4_BYTE_3(ptr)
#define LOG_DATA_16_BYTE_3(ptr)			LOG_DATA_8_BYTE_3(ptr);  LOG_DATA_8_BYTE_3(ptr)
#define LOG_DATA_32_BYTE_3(ptr)			LOG_DATA_16_BYTE_3(ptr); LOG_DATA_16_BYTE_3(ptr)
#define LOG_DATA_64_BYTE_3(ptr)			LOG_DATA_32_BYTE_3(ptr); LOG_DATA_32_BYTE_3(ptr)
#define LOG_DATA_128_BYTE_3(ptr)		LOG_DATA_64_BYTE_3(ptr); LOG_DATA_64_BYTE_3(ptr)
#define LOG_DATA_256_BYTE_3(ptr)		LOG_DATA_128_BYTE_3(ptr);LOG_DATA_128_BYTE_3(ptr)
#define LOG_DATA_512_BYTE_3(ptr)		LOG_DATA_256_BYTE_3(ptr);LOG_DATA_256_BYTE_3(ptr)
#define LOG_DATA_768_BYTE_3(ptr)		LOG_DATA_512_BYTE_3(ptr);LOG_DATA_256_BYTE_3(ptr)
#define LOG_DATA_1K_BYTE_3(ptr)			LOG_DATA_512_BYTE_3(ptr);LOG_DATA_512_BYTE_3(ptr)
#define LOG_DATA_1_5K_BYTE_3(ptr)		LOG_DATA_1K_BYTE_3(ptr);LOG_DATA_512_BYTE_3(ptr)

//**********************************************************
//			Faster Logging Function (4 clock cycle)
//**********************************************************

#define LOG_DATA_2_BYTE_4(ptr)			LOG_DATA_1_BYTE_4_CYCLE(ptr);  LOG_DATA_1_BYTE_4_CYCLE(ptr)
#define LOG_DATA_4_BYTE_4(ptr)			LOG_DATA_2_BYTE_4(ptr);  LOG_DATA_2_BYTE_4(ptr)
#define LOG_DATA_8_BYTE_4(ptr)			LOG_DATA_4_BYTE_4(ptr);  LOG_DATA_4_BYTE_4(ptr)
#define LOG_DATA_16_BYTE_4(ptr)			LOG_DATA_8_BYTE_4(ptr);  LOG_DATA_8_BYTE_4(ptr)
#define LOG_DATA_32_BYTE_4(ptr)			LOG_DATA_16_BYTE_4(ptr); LOG_DATA_16_BYTE_4(ptr)
#define LOG_DATA_64_BYTE_4(ptr)			LOG_DATA_32_BYTE_4(ptr); LOG_DATA_32_BYTE_4(ptr)
#define LOG_DATA_128_BYTE_4(ptr)		LOG_DATA_64_BYTE_4(ptr); LOG_DATA_64_BYTE_4(ptr)
#define LOG_DATA_256_BYTE_4(ptr)		LOG_DATA_128_BYTE_4(ptr);LOG_DATA_128_BYTE_4(ptr)
#define LOG_DATA_512_BYTE_4(ptr)		LOG_DATA_256_BYTE_4(ptr);LOG_DATA_256_BYTE_4(ptr)
#define LOG_DATA_768_BYTE_4(ptr)		LOG_DATA_512_BYTE_4(ptr);LOG_DATA_256_BYTE_4(ptr)
#define LOG_DATA_1K_BYTE_4(ptr)			LOG_DATA_512_BYTE_4(ptr);LOG_DATA_512_BYTE_4(ptr)
#define LOG_DATA_1_5K_BYTE_4(ptr)		LOG_DATA_1K_BYTE_4(ptr);LOG_DATA_512_BYTE_4(ptr)








