/*

        󑐋Z@ʃfBXvC̉p

        Copyright 2009-2011, Yoshi Hiroshi.

*/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdio.h>
#include <string.h>
#include "uDisp.h"
#include "uDispControl.h"
#include "simpleUart.h"
#include "switch.h"
#include "scope.h"
#include "logCommand.h"

#define CHANNEL_HEIGHT          17
#define GRAPH_HEIGHT            9
#define GRAPH_OFFSET            16

char logiScoTitle[] PROGMEM = "Portable Logi-Sco AVR";
char logiScoForm1[] PROGMEM = "%d";
char logiScoForm2[] PROGMEM = "%3dn";
char logiScoForm3[] PROGMEM = "%3du";
char logiScoForm4[] PROGMEM = "%3dm";
char logiScoForm5[] PROGMEM = "%4d";
char logiScoForm6[] PROGMEM = "S/DIV";
char logiScoForm7[] PROGMEM = "S/Smpl";

volatile int dispDataStartIndex = 0;

void setNextCaptureTimer(int time)
{
    lcb.captureTimer = time * 100;
}

int getDataPerDiv()
{
    long result = lcb.timebase;
    if(lcb.timebaseUnit != lcb.sampleSpeedUnit) {
        result *= 1000;
    }
    result /= lcb.sampleSpeed;
    return result;
}

void drawWave(unsigned int sx, unsigned int ex, unsigned int inx, char prev, char curr)
{
    unsigned char sy = 0;
    unsigned char ey;
    if(prev) {
        sy = inx * CHANNEL_HEIGHT + GRAPH_OFFSET;
    } else {
        sy = inx * CHANNEL_HEIGHT + GRAPH_OFFSET + GRAPH_HEIGHT;
    }
    if(curr) {
        ey = inx * CHANNEL_HEIGHT + GRAPH_OFFSET;
    } else {
        ey = inx * CHANNEL_HEIGHT + GRAPH_OFFSET + GRAPH_HEIGHT;
    }
    uDispDrawLine(sx, sy, ex, sy);
    if(sy != ey) {
        uDispDrawLine(ex, sy, ex, ey);
    }
}

void drawBar(unsigned int sx, unsigned int ex, unsigned int inx, char prevL, char prevH, char currL, char currH)
{
    unsigned int sy = inx * CHANNEL_HEIGHT + GRAPH_OFFSET + GRAPH_HEIGHT;
    unsigned int ey = inx * CHANNEL_HEIGHT + GRAPH_OFFSET + GRAPH_HEIGHT;
    if(!prevL) {        // no High
        // sy <= Low
    } else if(prevH) {  // no Low
        // sy <= High
        sy -= GRAPH_HEIGHT;
    } else {
    }
    if(!currL) {        // no High
        // ey <= Low
    } else if(currH) {  // no Low
        // ey <= High
        ey -= GRAPH_HEIGHT;
    } else {
        if(sy == ey) {
            sy -= GRAPH_HEIGHT;
        }
    }
    uDispDrawLine(ex, sy, ex, ey);
}

#define NUM_BOUND           20

void dispScope()
{
    unsigned int sx = 1;
    unsigned int ex;
    unsigned int dInxCurr = dispDataStartIndex;
    unsigned int dInxPrev = 0;
    unsigned int dataMax = lcb.buffPtr - lcb.buff;
    volatile unsigned char prevH = 0xFF;
    volatile unsigned char prevL = 0;
    volatile unsigned char currH = 0xFF;
    volatile unsigned char currL = 0;
    unsigned int ixStep = 1;
    unsigned int horizIx = 0;
    int numPerDiv = getDataPerDiv();
    if(numPerDiv < NUM_BOUND) {
        ixStep = NUM_BOUND / numPerDiv;
    }
    uDispSetColourInt(MENU_GRAPH_LINE_COLOUR);
    while(horizIx < 128) {
        ex = horizIx + 1;
        if(ex > 127) ex = 127;
        if(numPerDiv <= NUM_BOUND) {
            currL |= logBuffer[dInxCurr];
            if(horizIx > 0) {
                for(unsigned char bInx = 0; bInx < 6; bInx++) {
                    drawWave(sx, ex, bInx, prevL & _BV(bInx), currL & _BV(bInx));
                }
            }
        } else {
            if(horizIx > 0) {
                for(int inx = dInxPrev + 1; inx <= dInxCurr; inx++) {
                    currH &= logBuffer[inx];
                    currL |= logBuffer[inx];
                }
                for(unsigned char bInx = 0; bInx < 6; bInx++) {
                    drawBar(sx, ex, bInx, prevL & _BV(bInx), prevH & _BV(bInx), currL & _BV(bInx), currH & _BV(bInx));
                }
            } else {
                currH = logBuffer[0];
                currL = logBuffer[0];
            }
        }
        sx = ex;
        horizIx += ixStep;
        prevL = currL;
        currL = 0;
        dInxPrev = dInxCurr;
        if(numPerDiv > NUM_BOUND) {
            prevH = currH;
            currH = 0xFF;
            dInxCurr = (int)(((long)numPerDiv * (long)horizIx) / NUM_BOUND);
        } else {
            dInxCurr++;
        }
        if(dInxPrev > dataMax) break;
        if(IS_KEY_BREAK_INT) break;
    }
}

void dispSampleSpeed()
{
    sprintf_P(strWork, logiScoForm1, lcb.timebase);
    uDispSetColourInt(MENU_COLOUR);
    uDispSetFontSize(UDISP_FONT_SIZE_SMALL);
// timebase
    switch(lcb.timebaseUnit)
    {
    case 0:
        sprintf_P(strWork, logiScoForm2, lcb.timebase);
        break;
    case 1:
        sprintf_P(strWork, logiScoForm3, lcb.timebase);
        break;
    case 2:
        sprintf_P(strWork, logiScoForm4, lcb.timebase);
        break;
    case 3:
        sprintf_P(strWork, logiScoForm5, lcb.timebase);
        break;
    }
    uDispDrawText(15, 0, strWork);
    uDispDrawText_P(15, 4, logiScoForm6);
// sample period
    switch(lcb.sampleSpeedUnit)
    {
    case 0:
        sprintf_P(strWork, logiScoForm2, lcb.sampleSpeed);
        break;
    case 1:
        sprintf_P(strWork, logiScoForm3, lcb.sampleSpeed);
        break;
    case 2:
        sprintf_P(strWork, logiScoForm4, lcb.sampleSpeed);
        break;
    case 3:
        sprintf_P(strWork, logiScoForm5, lcb.sampleSpeed);
        break;
    }
    uDispDrawText(15, 11, strWork);
    uDispDrawText_P(15, 15, logiScoForm7);
}

void dispScreen()
{
    uDispErase();
    uDispSetColourInt(MENU_SCOPE_BACK_COLOUR);
    uDispDrawRectangle(UDISP_RECT_FILL_ON, 0, 13, 127, 113);
    uDispSetColourInt(MENU_COLOUR);
    uDispSetFontSize(UDISP_FONT_SIZE_SMALL);
    uDispDrawText_P(0, 0, logiScoTitle);
    dispSampleSpeed();

// horizontal line
    uDispSetColourInt(MENU_SEPALATE_LINE_COLOUR);
    for(char inx = 0; inx < 7; inx++)
    {
        uDispDrawLine(0,inx * 17 + 12, 127, inx * 17 + 12);
    }
    uDispSetColourInt(COLOUR_BLUE);
//    uDispDrawRectangle(UDISP_RECT_FILL_OFF, 0, 11, 127, 115);
    uDispDrawLine(0,11, 127, 11);
    uDispDrawLine(0,115, 127, 115);
// vertical line
    uDispSetColourInt(MENU_VERTICAL_LINE_COLOUR);
    for(int inx = 0; inx < (128 / NUM_BOUND); inx++)
    {
        uDispDrawLine(inx * NUM_BOUND, 12, inx * NUM_BOUND, 114);
    }
}

void dispCaptureData(unsigned char init)
{
    dispScreen();
    CLEAR_KEY_BREAK_INT;
    CLEAR_KEY_BREAK;
    purgeSwitchReq();
    enableKeyBreak(2);
    if(!IS_KEY_BREAK_INT) {
        uDispSetTextBackground(1);
        dispScope();
    }
    enableKeyBreak(0);
    if(IS_KEY_BREAK_INT) {
        CLEAR_KEY_BREAK_INT;
        waitForAllKeyRelease();
        SET_KEY_BREAK;
    }
    if(lcb.captureMode == 0) {
        setNextCaptureTimer(lcb.holdoffTime);
    }
}

char scopeOperation()
{
    char result = 0;
    if(SW_IS_PUSH(SW_UP)) {
        SW_ACK_PUSH(SW_UP);
        if(lcb.sampleSpeedIndex < SAMPLE_SPEED_INDEX_MAX) {
            lcb.timebaseIndex++;
            if(lcb.timebaseIndex > TIMEBASE_INDEX_MAX) {
                lcb.timebaseIndex = TIMEBASE_INDEX_MAX;
            }
            lcb.timebase = timebaseTable[lcb.timebaseIndex];
            lcb.timebaseUnit = timebaseUnitTable[lcb.timebaseIndex];
            lcb.sampleSpeedIndex++;
            lcb.sampleSpeed = timebaseTable[lcb.sampleSpeedIndex];
            lcb.sampleSpeedUnit = timebaseUnitTable[lcb.sampleSpeedIndex];
            dispSampleSpeed();
        }
        result = 1;
        dispDataStartIndex = 0;
    }
    if(SW_IS_PUSH(SW_DOWN)) {
        SW_ACK_PUSH(SW_DOWN);
        if(lcb.sampleSpeedIndex > SAMPLE_SPEED_INDEX_LOW) {
            lcb.timebaseIndex--;
            if(lcb.timebaseIndex < TIMEBASE_INDEX_LOW) {
                lcb.timebaseIndex = TIMEBASE_INDEX_LOW;
            }
            lcb.timebase = timebaseTable[lcb.timebaseIndex];
            lcb.timebaseUnit = timebaseUnitTable[lcb.timebaseIndex];
            lcb.sampleSpeedIndex--;
            lcb.sampleSpeed = timebaseTable[lcb.sampleSpeedIndex];
            lcb.sampleSpeedUnit = timebaseUnitTable[lcb.sampleSpeedIndex];
            dispSampleSpeed();
        }
        result = 1;
        dispDataStartIndex = 0;
    }
    if(SW_IS_PUSH(SW_ZOOMOUT)) {
        SW_ACK_PUSH(SW_ZOOMOUT);
        if(lcb.timebaseIndex < TIMEBASE_INDEX_MAX) {
            if(lcb.timebaseIndex < lcb.sampleSpeedIndex + 7) {
                lcb.timebaseIndex++;
                lcb.timebase = timebaseTable[lcb.timebaseIndex];
                lcb.timebaseUnit = timebaseUnitTable[lcb.timebaseIndex];
                dispSampleSpeed();
            }
        }
        result = 2;
    }
    if(SW_IS_PUSH(SW_ZOOMIN)) {
        SW_ACK_PUSH(SW_ZOOMIN);
        if(lcb.timebaseIndex > TIMEBASE_INDEX_LOW) {
            if(lcb.sampleSpeedIndex + 1 < lcb.timebaseIndex) {
                lcb.timebaseIndex--;
                lcb.timebase = timebaseTable[lcb.timebaseIndex];
                lcb.timebaseUnit = timebaseUnitTable[lcb.timebaseIndex];
                dispSampleSpeed();
            }
        }
        result = 2;
    }
    if(SW_IS_PUSH(SW_LEFT)) {
        SW_ACK_PUSH(SW_LEFT);
        result = 2;
        dispDataStartIndex -= (getDataPerDiv() * (lcb.scrollWidth + 1));
        if(dispDataStartIndex < 0) dispDataStartIndex = 0;
    }
    if(SW_IS_PUSH(SW_RIGHT)) {
        SW_ACK_PUSH(SW_RIGHT);
        result = 2;
        dispDataStartIndex += (getDataPerDiv() * (lcb.scrollWidth + 1));
    }
    return result;
}

